@@ -6,8 +6,8 @@ from kafi.fs.local.local_producer import LocalProducer
 #
 
 class Local(FS):
-    def __init__(self, config_str):
-        super().__init__("locals", config_str, ["local"], ["schema_registry"])
+    def __init__(self, config_str_or_dict):
+        super().__init__("locals", config_str_or_dict, ["local"], ["schema_registry"])
     
     #
 
