@@ -1,5 +1,4 @@
 from kafi.serializer import Serializer
-from kafi.schemaregistry import SchemaRegistry
 
 # Constants
 
@@ -10,11 +9,13 @@ class StorageProducer(Serializer):
     def __init__(self, storage_obj, topic, **kwargs):
         self.storage_obj = storage_obj
         #
+        super().__init__(storage_obj.schema_registry_config_dict)
+        #
         self.topic_str = topic
         #
         (self.key_type_str, self.value_type_str) = storage_obj.get_key_value_type_tuple(**kwargs)
         #
-        (self.key_schema_str, self.value_schema_str, self.key_schema_id_int, self.value_schema_id_int) = self.get_key_value_schema_tuple(**kwargs)
+        (self.key_schema_str_or_dict, self.value_schema_str_or_dict, self.key_schema_id_int, self.value_schema_id_int) = self.get_key_value_schema_tuple(**kwargs)
         #
         self.keep_partitions_bool = kwargs["keep_partitions"] if "keep_partitions" in kwargs else False
         #
@@ -25,11 +26,6 @@ class StorageProducer(Serializer):
         self.written_counter_int = 0
         #
         self.schema_hash_int_generalizedProtocolMessageType_dict = {}
-        #
-        if "schema.registry.url" in self.storage_obj.schema_registry_config_dict:
-            self.schemaRegistry = SchemaRegistry(self.storage_obj.schema_registry_config_dict, self.storage_obj.kafi_config_dict)
-        else:
-            self.schemaRegistry = None
 
     #
 
@@ -49,10 +45,10 @@ class StorageProducer(Serializer):
     # Helpers
 
     def get_key_value_schema_tuple(self, **kwargs):
-        key_schema_str = kwargs["key_schema"] if "key_schema" in kwargs else None
-        value_schema_str = kwargs["value_schema"] if "value_schema" in kwargs else None
+        key_schema_str_or_dict = kwargs["key_schema"] if "key_schema" in kwargs else None
+        value_schema_str_or_dict = kwargs["value_schema"] if "value_schema" in kwargs else None
         #
         key_schema_id_int = kwargs["key_schema_id"] if "key_schema_id" in kwargs else None
         value_schema_id_int = kwargs["value_schema_id"] if "value_schema_id" in kwargs else None
         #
-        return (key_schema_str, value_schema_str, key_schema_id_int, value_schema_id_int)
+        return (key_schema_str_or_dict, value_schema_str_or_dict, key_schema_id_int, value_schema_id_int)
